﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using StructureMap;
using StructureMap.Pipeline;
using Services;
using Library.Logging;
using Library.Structuremap;
using Core.IGateways;
using Core.Entities;
using Core.Dtos;

namespace Assembler
{ 
    /// <summary>
    /// Bootstrapper class is used to initialize the application on start
    /// </summary>
    public static class Bootstrapper
    {
        /// <summary>
        /// Amoung the initilization steps are : configure IoC library, configure AutoMapper
        /// </summary>
        /// <param name="useProxyServices"></param>
        /// <param name="ormContextLifecycle"></param>
        /// <param name="sessionLifecycle"></param>
        public static void Init(InstanceScope ormContextLifecycle, InstanceScope sessionLifecycle = InstanceScope.HttpSession)
        {
            ObjectFactory.Initialize(x =>
            {                
                x.For<CurrentContext>().LifecycleIs(StructuremapHelper.GetLifecycle(sessionLifecycle));                
                                                                                               
                x.AddRegistry(new RepositoryRegistry(ormContextLifecycle));
                x.AddRegistry<ServiceRegistry>();
                x.AddRegistry<GatewayRegistry>();
            });

            AutoMapperConfig.CreateMappings();            
        }

        public static T Resolve<T>() where T : class
        {
            return (T) ObjectFactory.TryGetInstance(typeof (T));
        }        
    }
}