﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace Core.Queries
{
    /// <summary>
    /// BaseQuery class is used to implement the Query Object Pattern
    /// It used a custom class LiteQuery. You can find it on codeplex
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public abstract class BaseQuery<T>
    {
        const int DefaultPageSize = 2;

        public BaseQuery()
        {
            PageSize = DefaultPageSize;
            SortAscending = true;
        }

        public bool ComputePagesCount { get; set; }
        public int PageSize { get; set; }
        public int PageNumber { get; set; }
        public string SortColumn { get; set; }
        public bool SortAscending { get; set; }

        public bool PagingEnabled
        {
            get
            {
                return PageSize > 0;
            }
            set
            {
                if (value)
                {
                    if (PageSize == 0)
                        PageSize = DefaultPageSize;
                }
                else
                    PageSize = 0;
            }
        }       

        public virtual Expression<Func<T, bool>> ToExpression()
        {            
            return x => true;
        }
    }
}
