﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Core.Dtos;
using Core.Entities;
using Core.IServices;
using Core.Messages;
using Core.Queries;
using Library.Helpers;

namespace Front.Mvc.Controllers
{
    public abstract class CrudBaseController<TId, TEntity, TDto, TQuery> : BaseController
        where TEntity : BaseEntity<TId>
        where TDto : BaseDto<TId>
        where TQuery : BaseQuery<TEntity>
    {        
        protected ICrudBaseService<TId, TEntity, TDto, TQuery> Service;

        public CrudBaseController(ICrudBaseService<TId, TEntity, TDto, TQuery> Service)
        {
            this.Service = Service;
        }

        [HttpPost]
        public virtual JsonResult Find(TQuery query)
        {
            ListResponse<TDto> response = Service.FindBy(query);
            return Json(response);
        }

        public virtual JsonResult FindById(string id)
        {
            BaseResponse<TDto> response = Service.FindById(id);
            return Json(response, JsonRequestBehavior.AllowGet);
        }

        [HttpPost]
        protected virtual JsonResult GenericEdit(TDto dto)
        {
            BaseResponse response = null;
            ClearInvalidErrors();

            if (ModelState.IsValid)
            {                
                if (Convert.ToInt32(dto.IdValue) > 0)
                {
                    response = Service.Edit(new BaseRequest<TDto>()
                    {
                        Value = dto
                    });
                }
                else
                {
                    response = Service.Add(new BaseRequest<TDto>()
                    {
                        Value = dto
                    });
                }
            }
            else
            {
                response = new BaseResponse();
                response.Success = false;
            }

            return Json(response);
        }

        [HttpPost]
        public virtual ActionResult Delete(string id)
        {
            BaseResponse response = Service.Delete(id);
            return Json(response);
        }

        public virtual FileStreamResult Export(TQuery query, string expType)
        {
            ListResponse<TDto> response = Service.FindBy(query);
            ColumnsFilter columsFilter = GetExportColumnsList();
            return ExportHelper.GenerateFileStreamResult<TDto>(response.List, expType, columsFilter);
        }

        protected abstract ColumnsFilter GetExportColumnsList();        

        protected virtual void ClearInvalidErrors()
        {
            this.ModelState["Id"].Errors.Clear();            
        }
    }
}
