﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.IO;
using Library.Helpers;
using Core.Entities;
using Core.IServices;
using Core.Queries;
using Core.IGateways;
using Core.Messages;
using Library.Web;
using Core.Dtos;

namespace Front.Mvc.Controllers
{   
    public class PolicyController : CrudBaseController<int, Policy, PolicyDto, PolicyQuery>
    {
        IPolicyService PolicyService;              

        public PolicyController(IPolicyService PolicyService) : base(PolicyService)
        {
            this.PolicyService = PolicyService;        
        }

        public ActionResult Index()
        {
            PolicyQuery query = new PolicyQuery();
            ListResponse<PolicyDto> response = PolicyService.FindBy(query);
            return View("Index", response.List);
        }

        public JsonResult Edit(MotorPolicyDto policyDto, PersonDto personDto)
        {
            policyDto.Person = personDto;
            return base.GenericEdit(policyDto);
        }

        protected override ColumnsFilter GetExportColumnsList()
        {
            ColumnsFilter columsFilter = new ColumnsFilter();
            columsFilter.AddColumn("Id");
            columsFilter.AddColumn("Key");
            columsFilter.AddColumn("Value");

            return columsFilter;
        }

        public FileStreamResult Document(string id)
        {
            Stream fileStream = PolicyService.GenerateDocument(id);
            HttpContext.Response.AddHeader("content-disposition", "attachment; filename=policy.pdf");
            return new FileStreamResult(fileStream, "application/pdf");
        }

        protected override void ClearInvalidErrors()
        {
            this.ModelState["Id"].Errors.Clear();
            this.ModelState["PersonID"].Errors.Clear();
        }
    }
}
