﻿(function ($) {
    jQuery.fn.sortableheader = function (properties) {
        this.each(function () {
            var grid = this;
            $("th a", this).css("cursor", "pointer");
            $("th[data-column=" + properties.defaultSortColumn + "]", this).addClass("asc").append('<i class="icon-arrow-down pull-right"></i>');

            $("th[data-column]", this).click(function () {
                var hasAsc = $(this).hasClass("asc");
                var hasDesc = $(this).hasClass("desc");
                $("th i").remove();
                $("th i").remove();
                $("th", grid).removeClass("asc").removeClass("desc");

                if (hasAsc)
                    $(this).addClass("desc").append('<i class="icon-arrow-up pull-right"></i>')
                if (hasDesc || (hasAsc == false && hasDesc == false))
                    $(this).addClass("asc").append('<i class="icon-arrow-down pull-right"></i>')

                properties.onChange($(this).attr("data-column"), $(this).hasClass("asc"));
            })

        });
    }

    $.fn.ajaxgrid = function (method, properties) {
        switch (method) {
            case "init":
                methods.init(this, properties);
                break;
            case "config":
                methods.setProperties(this, properties);
                break;
            case "bind":
                var defaults = { sortColumn: $(this).data("sortColumn"), sortAscending: true, pageNumber: 1, computePagesCount: true };
                var properties = $.extend({}, defaults, properties);
                this.each(function () {
                    methods.bindGrid(this, properties);
                });
                break;

            case "export":
                methods.exportFile(this);
                break;
        }
    }

    var methods =
    {
        setProperties: function (current, properties) {
            var existingProperties = $(current).data("properties");
            properties = $.extend({}, existingProperties, properties);
            $(current).data("properties", properties);
        },

        init: function (current, properties) {
            var existingProperties = $(current).data("properties");
            properties = $.extend({}, existingProperties, properties);
            if(properties.parent)
            {
                properties.clientFormSelector = properties.parent + " " + ".form-search";
                properties.clientExportSelector = properties.parent + " " + ".export";
                properties.templateSelector = properties.parent + " " + " .template";
            }

            var e = { properties: {} }
            if (typeof (properties.onInitializing) == "function")
                properties.onInitializing(e);
            properties = $.extend({}, properties, e.properties);

            $(current).data("properties", properties);
            $(current).data("sortColumn", properties.defaultSortColumn);

            var gridId = $(current).attr("id");

            var divPaging = $("<div/>");
            divPaging.attr("id", gridId + "Paging");
            divPaging.addClass("commands");
            current.after(divPaging);

            if (!$(this).hasClass("grid"))
                $(this).addClass("grid");

            current.each(function () {
                $(this).sortableheader({ defaultSortColumn: properties.defaultSortColumn, onChange: function (sortColumn, sortAscending) {
                    $(current).data("sortColumn", sortColumn);
                    $(current).data("sortAscending", sortAscending);
                    methods.bindGrid(current, { sortColumn: sortColumn, sortAscending: sortAscending, pageNumber: 1, computePagesCount: false });
                }
                });
            });

        if (properties.clientExportSelector)
            $(properties.clientExportSelector).click(function () { $(current).ajaxgrid('export'); });
        },

        processTemplate: function (current, properties) {
            var templateSelector = $(current).data("properties").templateSelector;
            var template = Hogan.compile($(templateSelector).html());
            $('tbody', current).html(template.render(properties));
        },

        enablePaging: function (current, properties) {
            var gridId = $(current).attr("id");
            var divPagingId = "#" + $(current).attr("id") + "Paging";

            var pagesCount;
            if (properties.computePagesCount) {
                $(current).data("pagesCount", properties.PagesCount);
                pagesCount = properties.PagesCount;
            }
            else
                pagesCount = $(current).data("pagesCount");

            if (pagesCount == 0) pagesCount = 1;
            
            var optionsPaging = {
                currentPage: properties.pageNumber,
                totalPages: pagesCount,
                onPageChanged: function (e, oldPage, newPage) {
                    $(current).data("pageNumber", newPage);
                    methods.bindGrid(current, { pageNumber: newPage, computePagesCount: false });
                },
                shouldShowPage: function (type, page, current) { return true; }
            }
            $(divPagingId).bootstrapPaginator(optionsPaging);
        },

        bindGrid: function (current, properties) {
            var defaults = { computePagesCount: false, pageNumber: $(current).data("pageNumber") };
            defaults = $.extend({}, { computePagesCount: false, pageNumber: $(current).data("pageNumber") }, $(current).data("properties"));
            var properties = $.extend({}, defaults, properties);

            var sortColumn = $(current).data("sortColumn");
            var sortAscending = $(current).data("sortAscending");
            var pageNumber = properties.pageNumber;
            var computePagesCount = properties.computePagesCount;
            var onSelect = properties.onSelect;
            var selectUrl = properties.selectUrl;
            var form = $(properties.clientFormSelector);

            var gridId = $(current).attr("id");

            var pagingSortingData = { PageNumber: pageNumber, SortColumn: sortColumn, SortAscending: sortAscending, ComputePagesCount: computePagesCount }
            var requestData = $.extend({}, $(form).serializeToJson(), pagingSortingData);
            var e = { data: {} }

            if (typeof (onSelect) == "function")
                onSelect(e)
            var requestData = $.extend({}, requestData, e.data);

            $.ajax({
                type: "POST",
                url: selectUrl,
                data: requestData,
                dataType: "json",
                contenType: "json",
                error: function (jqXHR, textStatus, errorThrown) { $.notify(errorThrown); },
                success: function (data) {
                    try {
                        if (typeof (properties.onLoading) == "function")
                            properties.onLoading(data);

                        var mergeProperties = $.extend({}, properties, data);
                        methods.processTemplate(current, mergeProperties);
                        if (mergeProperties.enablePaging) 
                            methods.enablePaging(current, mergeProperties);

                        if (typeof (mergeProperties.onLoad) == "function")
                            mergeProperties.onLoad({ id: gridId });

                    } catch (e) {
                        $.notify(e);
                    }
                }
            });
        },

        exportFile: function (current) {
            var properties = $(current).data("properties");
            var form = $(properties.clientFormSelector);
            var url = properties.exportUrl
            var onExport = properties.onExport;
            var sortColumn = $(current).data("sortColumn");
            var sortAscending = $(current).data("sortAscending");

            var requestData = $.extend({}, $(form).serializeToJson(), { PageNumber: 1, SortColumn: sortColumn, SortAscending: sortAscending, ComputePagesCount: false, PageSize: 400, expType: "xls" });

            if (typeof (onExport) == "function") {
                var e = { data: {} }
                onExport(e)
                var requestData = $.extend({}, requestData, e.data);
            }

            $.downloadFile(url, "POST", requestData);
        }

    }
})(jQuery);