﻿(function ($) {
    // =============== plugin default options ===============

    var defaultOptions = {
        'ignoreOnValidation': ':hidden',
        'ajaxCache': false,
        'clearFormMethod': null,
        'onAdd': null,
        'onEdit': null,
        'contollerUrl' : null,
        'editUrl': null,
        'deleteUrl': null,
        'saveCallback': null,
        'findByIdUrl': null,
        'onLoading': null,
        'onBeforeSave': null,
        'extendedPostData': null,
        'enablePaging': true
    };

    // =============== plugin public methods ===============

    var methods = {
        init: function (opts) {
            if (this.size() == 0) return;
            var options = $.extend({}, defaultOptions, opts);
            var parent = this;

            setPluginOptions(parent, options);
            doInit(parent);
        },
        option: function (key, val) {
            var options = getPluginOptions(this);
            options[key] = val;
            setPluginOptions(this, options);
        }
    };

    // =============== plugin definition ===============

    $.fn.crud = function (method) {
        // Method calling logic        
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method ' + method + ' does not exist.');
        }
    }

    // =============== private methods - start ===============

    function getPluginOptions(parent) {
        return $(parent).data('crud-options');
    }

    function setPluginOptions(parent, opts) {
        $(parent).data('crud-options', opts);
    }

    function setOptionsFromDataAttributes(parent, options)
    {
        if(!options.contollerUrl)
            options.contollerUrl = $('.grid', parent).attr("data-controller");
        if (!options.defaultSortColumn)
            options.defaultSortColumn = $('.grid', parent).attr("data-sort-column");
    }

    function setOptionsDefaultUrls(parent, options)
    {     
        if (options.contollerUrl) {
            if (!options.editUrl)
                options.editUrl = options.contollerUrl + "/Edit";
            if (!options.deleteUrl)
                options.deleteUrl = options.contollerUrl + "/Delete";
            if (!options.findByIdUrl)
                options.findByIdUrl = options.contollerUrl + "/FindById";
            if (!options.selectUrl)
                options.selectUrl = options.contollerUrl + "/Find";
            if (!options.exportUrl)
                options.exportUrl = options.contollerUrl + "/Export";
        }
    }

    function doInit(parent) {
        var options = getPluginOptions(parent);
        setOptionsFromDataAttributes(parent, options);
        setOptionsDefaultUrls(parent, options);

        $('.form-search .submit', parent).click(function (event) {
            event.preventDefault();
            $(".form-edit", parent).css("display", "none"); $('.grid', parent).ajaxgrid('bind');
        });
        $(".add", parent).click(function () { add(parent); });
        $(".form-edit .submit", parent).click(function () { save(parent); });
        $(".form-edit .cancel", parent).click(function () { $(".form-edit", parent).css("display", "none"); });
        $(".form-edit", parent).css("display", "none");

        $('.grid', parent).ajaxgrid('init', {
            parent: "#" + $(parent).attr("id"),
            defaultSortColumn: options.defaultSortColumn,
            selectUrl: options.selectUrl,
            exportUrl: options.exportUrl,
            enablePaging: options.enablePaging
        });

        $('.grid', parent).ajaxgrid("config", {
            onLoad: function () {
                $(".grid a.edit", parent).click(function () { edit(parent, $(this).attr("data-key")); });
                $(".grid a.delete", parent).click(function () {
                    var button = this;
                    if(confirm("Are you sure?"))                    
                        { deleteItem(parent, $(button).attr("data-key")); }
                    
                });
                
                if (typeof (options.onLoad) == "function")
                    options.onLoad();
            },
            onLoading: options.onLoading            
        });
        $('.grid', parent).ajaxgrid('bind');

        $.ajaxSetup({
            cache: options.ajaxCache
        });
    }

    function add(parent) {
        var options = getPluginOptions(parent);
        $(".form-edit", parent).css("display", "block");
        $(".form-edit input:not(:button)", parent).val("");

        // clear errors
        $(".form-edit", parent).validate().resetForm();

        if (options.onAdd && typeof (options.onAdd) === "function")
            options.onAdd(parent);
    }

    function save(parent) {
        var options = getPluginOptions(parent);
        var form = $(".form-edit", parent).get(0);
        var isValid = true;

        // use jquery unobtrusive validation
        if (typeof ($(form).validate) == 'function') {
            if (options.ignoreOnValidation) {
                $(form).validate().settings.ignore = options.ignoreOnValidation;
            }
            else {
                $(form).validate().settings.ignore = '';
            }
            $(form).validate();
            isValid = $(form).valid();
        }

        if (isValid) {
            if (typeof (options.onBeforeSave && options.onBeforeSave) === 'function') {
                options.onBeforeSave();
            }

            $.ajax({
                type: "POST",
                url: options.editUrl,
                data: $.extend({}, $(form).serializeToJson(), options.extendedPostData),
                traditional: true,
                dataType: "json",
                error: function (jqXHR, textStatus, errorThrown) { $.notify(errorThrown); },
                success: function (data) {
                    try {
                        if (data.Success) {
                            if (options.clearFormMethod && typeof (options.clearFormMethod) === 'function') {
                                options.clearFormMethod();
                            } else {
                                $(form).find('input[type=text],input[type=password], textarea').val('');
                            }

                            $(".form-edit", parent).css("display", "none");
                            $('.grid', parent).ajaxgrid("bind");

                            if (options.saveCallback && typeof (options.saveCallback) === 'function')
                                options.saveCallback(parent);
                        }

                        $.notify(data);
                    } catch (e) {
                        $.notify(e);
                    }
                }
            });

        }
    }

    function deleteItem(parent, id) {
        var options = getPluginOptions(parent);
        $.ajax({
            type: "POST",
            url: options.deleteUrl,
            data: { id: id },
            contenType: "json",
            error: function (jqXHR, textStatus, errorThrown) { $.notify(errorThrown); },
            success: function (data) {
                try {
                    if (data.Success)
                        $('.grid', parent).ajaxgrid("bind");
                    $.notify(data);
                } catch (e) {
                    $.notify(e);
                }
            }
        });
    }

    function edit(parent, id) {
        var options = getPluginOptions(parent);

        $(".form-edit", parent).css("display", "block");
        $(".form-edit", parent).validate().resetForm();

        $.ajax({
            type: "GET",
            url: options.findByIdUrl,
            data: { id: id },
            contenType: "json",
            error: function (jqXHR, textStatus, errorThrown) { $.notify(errorThrown); },
            success: function (data) {
                try {
                    mapResponseValuesToUI(parent, data);

                    if (options.onEdit && typeof (options.onEdit) === "function")
                        options.onEdit($(".form-edit", parent), data);
                } catch (e) {
                    $.notify(e);
                }
            }
        });
    }

    function mapResponseValuesToUI(parent, data) {
        var type;
        for (var prop in data.Value) {
            var type = typeof (data.Value[prop]);
            if (type == "number" || type == "string") {
                $(".form-edit #" + prop, parent).val(data.Value[prop]);
            }
        }
    }

    // =============== private methods - end ===============

})(jQuery);