using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Reflection;
using System.Xml;
using System.IO;
using System.Xml.Serialization;
using System.Collections;
using System.Web;
using iTextSharp.text;
using iTextSharp.text.pdf;
using Library.Web.UI;
using Yogesh.ExcelXml;
using System.Web.Mvc;

namespace Library.Helpers
{
    /// <summary>
    /// ExportHelper is an utiliy class used to transform a tabular data (DataTable or List<object>) in a csv,pdf,excel file.
    /// </summary>
    public static class ExportHelper
    {
        public static Object ExecMethodAndReturnDataTable(string xmlRequest)
        {
            Object result = new Object();

            XmlDocument xmlDocument = new XmlDocument();
            xmlDocument.LoadXml(xmlRequest);

            XmlNode requestMethod = xmlDocument.SelectSingleNode("RootXML/Call/Request/Method");
            XmlNodeList requestParameters = xmlDocument.SelectNodes("RootXML/Call/Request/Method/Arguments/Argument");
             
            string requestMethodProject = requestMethod.Attributes["library"].Value;
            string requestMethodNamespace = requestMethod.Attributes["namespace"].Value;
            string requestMethodName = requestMethod.Attributes["name"].Value;
            string requestMethodType = requestMethod.Attributes["type"].Value;

            object[] paramValues = new object[requestParameters.Count];
            Type[] paramTypes = new Type[requestParameters.Count];

            Object instanceObject = new Object();

            for (int i = 0; i < requestParameters.Count; i++)
            {
                string passingMode = requestParameters[i].Attributes["passingmode"].Value;
                string isPrimary = requestParameters[i].Attributes["isprimary"].Value;
                switch (isPrimary)
                {
                    case "true":
                        {
                            Type parameterType = Type.GetType(requestParameters[i].Attributes["type"].Value);
                            switch (passingMode)
                            {
                                case "In":
                                    {
                                        //instanceObject = Activator.CreateInstance(parameterType);                                        
                                        paramValues[i] = Convert.ChangeType(requestParameters[i].LastChild.InnerText, parameterType);
                                    }
                                    break;
                                case "Out":
                                    parameterType = Type.GetType(requestParameters[i].Attributes["type"].Value + "&");
                                    break;
                                case "Ref":
                                    break;
                            }
                            paramTypes[i] = parameterType;
                        } break;
                    case "false":
                        {
                            string library = requestParameters[i].Attributes["library"].Value;
                            string nameSpace = requestParameters[i].Attributes["namespace"].Value;
                            string serializedXML = requestParameters[i].LastChild.InnerText;

                            instanceObject = Serialize.DeserializeObject(serializedXML, library, nameSpace);

                            paramTypes[i] = instanceObject.GetType();
                            paramValues[i] = instanceObject;
                        } break;
                }
            }            

            Type type = AppDomain.CurrentDomain.Load(requestMethodProject).GetType(requestMethodNamespace);
            MethodInfo method = type.GetMethod(requestMethodName, paramTypes);            

            object objectReturn = method.Invoke(null, paramValues);

            result = Convert.ChangeType(objectReturn, method.ReturnType);

            return result;
        }

        public static void ExportFile(byte[] data, Library.Web.UI.ExportFileType fileType, string fileName)
        {
            if (fileName == null || fileName.Trim().Length == 0)
                fileName = DateTime.Now.ToString("yyyyMMdd_HHmm") + "_list";

            System.Web.HttpContext.Current.Response.ClearContent();
            System.Web.HttpContext.Current.Response.ClearHeaders();
            System.Web.HttpContext.Current.Response.ContentType = "application/" + fileType.ToString();
            System.Web.HttpContext.Current.Response.AppendHeader("Content-Disposition", "attachment; filename=" + fileName + "." + fileType.ToString());

            System.Web.HttpContext.Current.Response.ContentEncoding = Encoding.Unicode;
            System.Web.HttpContext.Current.Response.Charset = "UTF-16";

            System.Web.HttpContext.Current.Response.BinaryWrite(data);
            System.Web.HttpContext.Current.Response.Flush();
            System.Web.HttpContext.Current.Response.Close();
        }

        public static void ExportFileText(string data, Library.Web.UI.ExportFileType fileType, string fileName)
        {
            if (fileName == null || fileName.Trim().Length == 0)
                fileName = DateTime.Now.ToString("yyyyMMdd_HHmm") + "_list";

            System.Web.HttpContext.Current.Response.ClearContent();
            System.Web.HttpContext.Current.Response.ClearHeaders();
            System.Web.HttpContext.Current.Response.ContentType = "text/xml"; 
            System.Web.HttpContext.Current.Response.AppendHeader("Content-Disposition", "attachment; filename=" + fileName + "." + fileType.ToString());
            System.Web.HttpContext.Current.Response.ContentEncoding = Encoding.UTF8; 

            System.Web.HttpContext.Current.Response.BinaryWrite(Encoding.UTF8.GetPreamble());
            System.Web.HttpContext.Current.Response.Write(data);
            System.Web.HttpContext.Current.Response.Flush();
            System.Web.HttpContext.Current.Response.Close();           
        }        

        public static void ExportDataToFile(
            string reportName, string methodInfoXml, string xslPath
            , Library.Web.UI.ExportFileType fileType, string listSeparator, string fileName
            , DataTable sourceTable
            , bool exectMethodWithReflaction
            )
        {
            DataTable inputTable;
            if (exectMethodWithReflaction)
                inputTable = (DataTable)ExecMethodAndReturnDataTable(methodInfoXml);
            else
                inputTable = sourceTable;

            DataTable filterTable = GetColumnsFilter(methodInfoXml);
            DataTable dtResult = GetResult(inputTable, filterTable);
            
            string result = "";

            switch (fileType)
            {
                case ExportFileType.xml:                                        
                    ExportFileText(GetXmlFromDataTable(dtResult), fileType, fileName);
                    return;
                case ExportFileType.csv:
                    ExportFileText(GetCSV(dtResult, listSeparator), fileType, fileName);
                    break;
                case ExportFileType.pdf:
                    ExportFile(GetPDF(reportName, dtResult), ExportFileType.pdf, fileName);
                    return;
                case ExportFileType.zip:
                    ExportFile(GetCSV_ZIP(dtResult, listSeparator), ExportFileType.zip, fileName);
                    return;
                case ExportFileType.xls:
                    ExportFile(GetExcel(dtResult), ExportFileType.xls, fileName);
                    return;
                default:
                    string xmlResult1 = GetXmlFromDataTable(dtResult);
                    result = XlsHelper.TransformOutput(xmlResult1, xslPath);
                    ExportFileText(result, fileType, fileName);
                    return;
            }           
        }

        public static DataTable GetResult(DataTable dtIn, DataTable columnsFilter)
        {
            DataTable dataTable = dtIn.Copy();
            if (columnsFilter == null || columnsFilter.Rows.Count == 0)
            {
                dataTable.TableName = "DataTable";
                return dataTable;
            }

            DataTable dtResult = new DataTable();

            for (int c = 0; c < dataTable.Columns.Count; )
            {
                bool exists = false;
                for (int i = 0; i < columnsFilter.Rows.Count; i++)
                    if (columnsFilter.Rows[i]["columnField"].ToString().ToLower().Equals(dataTable.Columns[c].ColumnName.ToLower()))
                    {
                        exists = true;
                        break;
                    }
                if (!exists)
                    dataTable.Columns.RemoveAt(c);
                else
                    c++;
            }

            ArrayList formats = new ArrayList();
            for (int i = 0; i < columnsFilter.Rows.Count; i++)
            {
                if (!dataTable.Columns.Contains(columnsFilter.Rows[i]["columnField"].ToString()))
                    continue;

                DataColumn dc = dtResult.Columns.Add();

                dc.ColumnName = columnsFilter.Rows[i]["columnField"].ToString();
                dc.Caption = columnsFilter.Rows[i]["columnName"].ToString();
                if(columnsFilter.Rows[i]["columnDataFormatString"].ToString() != "")
                    formats.Add(columnsFilter.Rows[i]["columnDataFormatString"].ToString());
                else
                    formats.Add(null);
                switch ((Library.Web.UI.Column.columnType)Enum.Parse(typeof(Library.Web.UI.Column.columnType), columnsFilter.Rows[i]["columnType"].ToString()))
                {
                    case Library.Web.UI.Column.columnType.Text:
                        dc.DataType = typeof(object);
                        break;
                    case Library.Web.UI.Column.columnType.Number:
                        dc.DataType = typeof(double);
                        break;
                    case Library.Web.UI.Column.columnType.Default:
                        dc.DataType = typeof(string);
                        break;
                    default:
                        break;
                }
            }

            foreach (DataRow dr in dataTable.Rows)
            {
                DataRow drn = dtResult.NewRow();
                for (int i = 0; i < dtResult.Columns.Count; i++)
                {
                    if (formats[i] == null)
                        drn[i] = dr[dtResult.Columns[i].ColumnName];
                    else
                        drn[i] = String.Format(
                            (string)formats[i]
                            , dr[dtResult.Columns[i].ColumnName]);
                }
                dtResult.Rows.Add(drn);
            }
            dtResult.TableName = "DataTable";

            return dtResult;
        }

        public static string GetXmlFromDataTable(DataTable dtResult)
        {
            StringWriter sw = new StringWriter();
            dtResult.WriteXml(sw, XmlWriteMode.WriteSchema, true);
            return sw.ToString();
        }

        private static string GetCSV(DataTable dtResult, string listSeparator)
        {
            StringBuilder textCSV = new StringBuilder(string.Empty);

            int nrCol = dtResult.Columns.Count;
            if (nrCol == 0) return textCSV.ToString();

            int nrRows = dtResult.Rows.Count;
            //string listSeparator = System.Threading.Thread.CurrentThread.CurrentCulture.TextInfo.ListSeparator;

            textCSV.Append('"' + dtResult.Columns[0].Caption + '"');
            for (int k = 1; k < nrCol; k++)
                textCSV.Append(listSeparator + '"' + dtResult.Columns[k].Caption + '"');
            textCSV.Append(System.Environment.NewLine);
            
            //aici sint datele
            for (int i = 0; i < nrRows; i++)
            {
                textCSV.Append('"' + dtResult.Rows[i][0].ToString() + '"');
                for (int j = 1; j < nrCol; j++)
                    textCSV.Append(listSeparator + '"' + dtResult.Rows[i][j].ToString() + '"');
                textCSV.Append(System.Environment.NewLine);
            }

            return textCSV.ToString();
        }

        private static byte[] GetPDF(string reportName, DataTable dtResult)
        {
            int nrRows = dtResult.Rows.Count;
            int nrCol = dtResult.Columns.Count;
            float tablePadding = 2.0f
                , tableSpacing = 2.0f, tableBorder = 1.0f
                , textWidth;
            string title = String.IsNullOrEmpty(reportName) ? "Raport" : reportName;
            title += String.Format(" ({0:dd.mm.yy hh:mm})", DateTime.Now);

            Document document = new Document(PageSize.A4, 0, 0, 50, 50);
            System.IO.MemoryStream msReport = new System.IO.MemoryStream();
            PdfWriter writer = PdfWriter.GetInstance(document, msReport);

            document.AddAuthor("Test");// Todo: De pus numele 
            document.AddSubject("Export to PDF");
            document.AddTitle(title);

            iTextSharp.text.Table datatable = new iTextSharp.text.Table(nrCol);
            datatable.Padding = tablePadding;
            datatable.Spacing = tableSpacing;
            datatable.DefaultCellBorderWidth = tableBorder;
            datatable.DefaultRowspan = 1;
            datatable.DefaultHorizontalAlignment = Element.ALIGN_CENTER;
            Chunk text = new Chunk(title, new Font(Font.UNDEFINED, 16.0f, Font.BOLD));
            Paragraph titleParagraph = new Paragraph(text);
            titleParagraph.SpacingBefore = 5.0f;
            titleParagraph.SpacingAfter = 15.0f;
            titleParagraph.IndentationLeft = 20.0f;
            //HeaderFooter footer = new HeaderFooter(new Paragraph("Pagina "), true);
            //footer.Border = Rectangle.NO_BORDER;
            //document.Footer = footer;

            Font headerFont = new Font(Font.UNDEFINED, 14.0f, Font.BOLD);
            Font bodyFont = new Font(Font.UNDEFINED, 12.0f);

            float[] maxColWidth = new float[nrCol];
            float sum = 0;

            iTextSharp.text.Cell headerCell;
            for (int j = 0; j < nrCol; j++)
            {
                text = new Chunk(dtResult.Columns[j].Caption.ToString(), headerFont);
                headerCell = new iTextSharp.text.Cell(text);
                headerCell.BackgroundColor = Color.LIGHT_GRAY;
                datatable.AddCell(headerCell);

                maxColWidth[j] = text.GetWidthPoint();
            }

            for (int i = 0; i < nrRows; i++)
                for (int j = 0; j < nrCol; j++)
                {
                    text = new Chunk(dtResult.Rows[i][j].ToString(), bodyFont);
                    datatable.AddCell(new iTextSharp.text.Cell(text));

                    textWidth = text.GetWidthPoint();
                    if (maxColWidth[j] < textWidth && textWidth < 300.0f)
                        maxColWidth[j] = textWidth;
                }

            for (int j = 0; j < nrCol; j++)
                sum += maxColWidth[j];

            float totalCellWidthProportion = 
                (document.PageSize.Width * datatable.Width / 100
                    - 2 * tablePadding * nrCol - tableBorder * (nrCol + 1)
                    - 2 * nrCol)
                / sum;
            // Pentru a evita cazurile in care fontul este mai mic de 4 fiind considerat ilizibil
            if (totalCellWidthProportion < 1 / 3) totalCellWidthProportion = 1 / 3;

            if (totalCellWidthProportion < 1)
            {
                headerFont.Size = (int)(headerFont.Size * totalCellWidthProportion);
                bodyFont.Size = (int)(bodyFont.Size * totalCellWidthProportion);
            }

            int[] widths = new int[nrCol];
            for (int j = 0; j < nrCol; j++)
                widths[j] = (int)(maxColWidth[j] * 100 / sum);

            datatable.SetWidths(widths);

            document.Open();
            document.Add(titleParagraph);
            document.Add(datatable);
            document.Close();

            return msReport.ToArray();
        }

        private static byte[] GetCSV_ZIP(DataTable dtResult, string listSeparator)
        {
            return CompressionHelper.CompressWithZip(
                GetCSV(dtResult, listSeparator),
                DateTime.Now.Year.ToString() + "." + DateTime.Now.Month.ToString() + "." + DateTime.Now.Day.ToString() + "-DataGrid" + ".csv");
        }

        private static byte[] GetExcel(DataTable dtResult)
        {
            MemoryStream ms = new MemoryStream();
            if(dtResult.Columns.Count ==0 && dtResult.Rows.Count == 0)
            {
                return ms.ToArray();
            }


            int maxWidth = 230, maxHeight = 65536, maxNrOfSheets = 200;
            int nrRowsUnwritten = dtResult.Rows.Count;
            ExcelXmlWorkbook book = new ExcelXmlWorkbook();
            book.Properties.Author = "Title";
            Worksheet sheet;

            int nrCol = Math.Min(maxWidth - 1, dtResult.Columns.Count)
                , nrRows;

            for (int currentSheet = 0; currentSheet < maxNrOfSheets; )
            {
                nrRows = Math.Min(maxHeight - 1, nrRowsUnwritten);

                sheet = book[currentSheet];
                sheet.Name = "Report " + currentSheet;

                Range range = new Range(sheet[0, 0], sheet[nrCol - 1, 0]);
                range.Font.Bold = true;

                // Copy the column names to the first row of the object array
                for (int col = 0; col < nrCol; col++)
                    sheet[col, 0].Value = dtResult.Columns[col].Caption;

                // Copy the values to the object array
                for (int col = 0; col < nrCol; col++)
                    for (int row = 0; row < nrRows; row++)
                        sheet[col, row + 1].Value = dtResult.Rows[dtResult.Rows.Count - nrRowsUnwritten + row][col].ToString();

                nrRowsUnwritten -= nrRows;
                if (nrRowsUnwritten > 0)
                {
                    book.InsertAfter(currentSheet);
                    currentSheet++;
                }
                else
                    break;
            }

            book.Export(ms);
            return ms.ToArray();
        }

        //TODO: Move in MVC library
        public static FileStreamResult GenerateFileStreamResult<T>(IList<T> list, string expType, ColumnsFilter colFilter)
        {

            DataTable table = ListHelper.ToDataTable<T>(list);
            return GenerateFileStreamResult(table, expType, colFilter);
        }

        public static FileStreamResult GenerateFileStreamResult(DataTable data, string expType, ColumnsFilter colFilter)
        {
            string fileName = DateTime.Now.Year.ToString() + "." + DateTime.Now.Month.ToString() + "." + DateTime.Now.Day.ToString() + "-DataGrid";
            ExportFileType fileType = EnumHelper<ExportFileType>.ToEnum(expType);

            DataTable formatedTable;
            if (colFilter == null)
                formatedTable = data;
            else
                formatedTable = GetResult(data, colFilter.ColumnTable);

            byte[] content = null;
            switch (fileType)
            {
                case ExportFileType.pdf:
                    content = GetPDF(fileName, formatedTable);
                    break;
                case ExportFileType.xls:
                    content = ExcelExport.GetExcelAsByteArray(formatedTable);
                    fileType = ExportFileType.xlsx;
                    //GetExcel(formatedTable);
                    break;
                case ExportFileType.csv:
                    content = Encoding.UTF8.GetBytes(GetCSV(formatedTable, ","));
                    break;
            }

            HttpContext.Current.Response.AddHeader("content-disposition", "attachment; filename=" + fileName + "." + fileType.ToString());
            Stream fileStream = StreamHelper.CreateStreamFromByteArray(content);
            return new FileStreamResult(fileStream, "application/" + fileType.ToString());
        }

        public static string ExecMethodAndReturnHtml(string methodInfoXml, string xslPath)
        {
            DataTable dtResult = (DataTable)ExecMethodAndReturnDataTable(methodInfoXml);
            string xmlResult = GetXmlFromDataTable(dtResult);
            string result = "";

            result = XlsHelper.TransformOutput(xmlResult, xslPath);

            return result;
        }

        public static string BuildRequestXML(RequestXMLMethodFilter filter, ColumnsFilter colFilter)
        {
            /*<RootXML>
             *  <Call id=\"0\">
             *      <Request>
             *          <Method name=\"GetImportFieldsPerSchema2\" namespace=\"DebtCollector.BLL.Imports.Imports\" type=\"System.Data.DataTable\" library=\"DebtCollector.BLL\">
             *              <Arguments>
             *              </Arguments>
             *          </Method>
             *      </Request>
             *      <Response>
             *          <Method>
             *          </Method>
             *      </Response>
             *  </Call>
             *</RootXML>*/
             
            XmlDocument xmlDoc = new XmlDocument();

            XmlNode xmlRootNode = xmlDoc.CreateNode("element", "RootXML", "");
            xmlDoc.AppendChild(xmlRootNode);

            XmlNode xmlCallNode = xmlDoc.CreateNode("element", "Call", "");
            AddAttributeToNode(xmlDoc, xmlCallNode, "id", "0");
            xmlRootNode.AppendChild(xmlCallNode);

            XmlNode xmlRequestNode = xmlDoc.CreateNode("element", "Request", "");
            xmlCallNode.AppendChild(xmlRequestNode);

            XmlNode xmlMethodNode = xmlDoc.CreateNode("element", "Method", "");
            AddAttributeToNode(xmlDoc, xmlMethodNode, "name", filter.MethodName);
            AddAttributeToNode(xmlDoc, xmlMethodNode, "namespace", filter.NameSpace);
            AddAttributeToNode(xmlDoc, xmlMethodNode, "type", filter.MethodReturnType);
            AddAttributeToNode(xmlDoc, xmlMethodNode, "library", filter.LibraryName);

            xmlMethodNode.InnerXml = filter.ParameterXML; 
            xmlRequestNode.AppendChild(xmlMethodNode);
            
            if (colFilter.ColumnNode != null) //Cristi 19.09.2008
            xmlRequestNode.AppendChild(xmlDoc.ImportNode(colFilter.ColumnNode, true));
            return xmlDoc.OuterXml;
        }

        public static DataTable GetColumnsFilter(string xmlRequest)
        {
            DataTable dtResult = new DataTable();
            dtResult.Columns.Add("columnField");
            dtResult.Columns.Add("columnName");
            dtResult.Columns.Add("columnType");
            dtResult.Columns.Add("columnDataFormatString");

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(xmlRequest);
            XmlNodeList columnNodes = xmlDoc.SelectNodes("RootXML/Call/Request/Columns/Column");
            
            for (int i = 0; i < columnNodes.Count; i++)
            {
                DataRow dr = dtResult.NewRow();

                dr["columnField"] = columnNodes[i].Attributes["columnField"].Value;
                dr["columnName"] = columnNodes[i].Attributes["columnName"].Value;
                dr["columnType"] = columnNodes[i].Attributes["columnType"].Value;
                dr["columnDataFormatString"] = columnNodes[i].Attributes["columnDataFormatString"].Value;

                dtResult.Rows.Add(dr);
            }

            return dtResult;
        }        

        public static void AddAttributeToNode(XmlDocument xmlDoc, XmlNode xmlNode, string attributeName, string attributeValue)
        {
            XmlAttribute xmlAttribute = xmlDoc.CreateAttribute(attributeName);
            xmlAttribute.Value = attributeValue;
            xmlNode.Attributes.Append(xmlAttribute);
        }
    }
            
    public enum ParameterPassingMode
    {
        In,
        Out,
        Ref
    }

    [Serializable]
    public class RequestXMLMethodFilter
    {
        string libraryName;
        string nameSpace;
        string methodName;
        string methodReturnType;
        string parameterXML;
        ArrayList parameterArray;

        public string LibraryName
        {
            get { return libraryName; }
            set { libraryName = value; }
        }

        public string NameSpace
        {
            get { return nameSpace; }
            set { nameSpace = value; }
        }

        public string MethodName
        {
            get { return methodName; }
            set { methodName = value; }
        }

        public string MethodReturnType
        {
            get { return methodReturnType; }
            set { methodReturnType = value; }
        }

        public string ParameterXML
        {
            get { return parameterXML; }
            set { parameterXML = value; }
        }

        public ArrayList ParameterArray
        {
            get { return parameterArray; }
            set { parameterArray = value; }
        }

        public RequestXMLMethodFilter()
        {
            ParameterArray = new ArrayList();

            XmlDocument xmlDoc = new XmlDocument();
            XmlNode columnNode = xmlDoc.CreateNode("element", "Arguments", "");
            xmlDoc.AppendChild(columnNode);

            ParameterXML = xmlDoc.OuterXml;
        }

        public void AddSystemParameter(string name, ParameterPassingMode passingmode, string type, string value)
        {
            /*<Argument name=\"schemaId\" passingmode=\"in\" isprimary=\"true\" type=\"System.Int32\">dfgsdfgs</Argument>*/

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(ParameterXML);

            XmlNode xmlNode = xmlDoc.SelectSingleNode("Arguments");
            XmlNode columnNode = xmlDoc.CreateNode("element", "Argument", "");

            AddAttributeToNode(xmlDoc, columnNode, "name", name);
            AddAttributeToNode(xmlDoc, columnNode, "type", type);
            AddAttributeToNode(xmlDoc, columnNode, "isprimary", "true");
            AddAttributeToNode(xmlDoc, columnNode, "passingmode", passingmode.ToString());

            columnNode.InnerXml = "<![CDATA[" + value + "]]>";
            xmlNode.AppendChild(columnNode);

            ParameterXML = xmlDoc.OuterXml;
            ParameterArray.Add(columnNode.OuterXml);
        }

        public void AddDefinedParameter(ParameterPassingMode passingmode, string serializedXML, string nameSpace, string library)
        {
            /*<Argument passingmode=\"in\" library=\"Reflection.Library.ContractsFilter\" type=\"System.Data.DataTable\" namespace=\"Reflection.Library\">
             * <?xml version="1.0" encoding="utf-16"?>
                <ContractsFilter xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
                  <contractID>2</contractID>
                  <contractorID>3</contractorID>
                  <ContractID>2</ContractID>
                  <ContractorID>3</ContractorID>
                </ContractsFilter>
             * </Argument>*/

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(ParameterXML);

            XmlNode xmlNode = xmlDoc.SelectSingleNode("Arguments");
            XmlNode columnNode = xmlDoc.CreateNode("element", "Argument", "");

            AddAttributeToNode(xmlDoc, columnNode, "library", library);
            AddAttributeToNode(xmlDoc, columnNode, "namespace", nameSpace);
            AddAttributeToNode(xmlDoc, columnNode, "type", "System.Data.DataTable");
            AddAttributeToNode(xmlDoc, columnNode, "isprimary", "false");
            AddAttributeToNode(xmlDoc, columnNode, "passingmode", passingmode.ToString());

            columnNode.InnerXml = "<![CDATA[" + serializedXML + "]]>";
            xmlNode.AppendChild(columnNode);

            ParameterXML = xmlDoc.OuterXml;
            ParameterArray.Add(columnNode.OuterXml);
        }

        /// <summary>
        /// Adds attributes to a certain node in an XmlDocument
        /// </summary>
        /// <param name="xmlDoc">XML Document in which to insert</param>
        /// <param name="xmlNode">Parent XmlNode</param>
        /// <param name="attributeName">Attribute Name</param>
        /// <param name="attributeValue">Attribute Value</param>
        public void AddAttributeToNode(XmlDocument xmlDoc, XmlNode xmlNode, string attributeName, string attributeValue)
        {
            XmlAttribute xmlAttribute = xmlDoc.CreateAttribute(attributeName);
            xmlAttribute.Value = attributeValue;
            xmlNode.Attributes.Append(xmlAttribute);
        }
    }

    public class ColumnsFilter
    {
        string columnsXML;
        DataTable columnTable;
        XmlNode columnNode;

        public string ColumnsXML
        {
            get { return columnsXML; }
            set { columnsXML = value; }
        }

        public DataTable ColumnTable
        {
            get { return columnTable; }
            set { columnTable = value; }
        }

        public XmlNode ColumnNode
        {
            get { return columnNode; }
            set { columnNode = value; }
        }

        public ColumnsFilter()
        {
            Initialize();
        }

        void Initialize()
        {
            ColumnTable = new DataTable();
            ColumnTable.Columns.Add("columnField");
            ColumnTable.Columns.Add("columnName");
            ColumnTable.Columns.Add("columnType");
            ColumnTable.Columns.Add("columnDataFormatString");

            XmlDocument xmlDoc = new XmlDocument();
            XmlNode columnNode = xmlDoc.CreateNode("element", "Columns", "");
            xmlDoc.AppendChild(columnNode);

            ColumnsXML = xmlDoc.OuterXml;
        }

        public void Clear()
        {
            //this methid should be optimize to clear the existing objects not creating new ones
            Initialize();
        }

        public void AddColumn(string columnField)
        {
            AddColumn(columnField, Library.Web.UI.Column.columnType.Default, columnField, "");
        }

        public void AddColumn(string columnField, Library.Web.UI.Column.columnType columnType, string columnName, string dataFormatString)
        {
            /*<Columns>
             *  <Column columnField="" columnName=""></Column>
             </Columns>*/

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(ColumnsXML);

            XmlNode xmlNode = xmlDoc.SelectSingleNode("Columns");
            XmlNode columnNode = xmlDoc.CreateNode("element", "Column", "");

            ExportHelper.AddAttributeToNode(xmlDoc, columnNode, "columnField", columnField);
            ExportHelper.AddAttributeToNode(xmlDoc, columnNode, "columnName", columnName);
            ExportHelper.AddAttributeToNode(xmlDoc, columnNode, "columnType", columnType.ToString());
            ExportHelper.AddAttributeToNode(xmlDoc, columnNode, "columnDataFormatString", dataFormatString);
            
            xmlNode.AppendChild(columnNode);

            DataRow dr = ColumnTable.NewRow();

            dr["columnField"] = columnField;
            dr["columnName"] = columnName;
            dr["columnType"] = columnType.ToString();
            dr["columnDataFormatString"] = dataFormatString;

            ColumnTable.Rows.Add(dr);

            ColumnsXML = xmlDoc.OuterXml;
            ColumnNode = xmlNode;
        }
    }

    public class Serialize
    {
        public static String SerializeObject(Object pObject)
        {
            XmlSerializer ser = new XmlSerializer(pObject.GetType());
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            System.IO.StringWriter writer = new System.IO.StringWriter(sb);
            ser.Serialize(writer, pObject);

            return sb.ToString();
        }

        public static Object DeserializeObject(String pXmlizedString, string library, string nameSpace)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(pXmlizedString.ToString());
            XmlNodeReader reader = new XmlNodeReader(doc.DocumentElement);
            XmlSerializer ser = new XmlSerializer(AppDomain.CurrentDomain.Load(library).GetType(nameSpace));
            object obj = ser.Deserialize(reader);

            return obj;
        }
    }

}