using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Text.RegularExpressions;
using System.Data;
using Library.Helpers;

namespace Library.Web.UI
{
    public class TableExport : CompositeControl, INamingContainer
    {
        #region Member variables

        public event EventHandler Exporting;

        private RequestXMLMethodFilter filter = new RequestXMLMethodFilter();
        private Columns columns = new Columns();

        private string excelImageUrl;
        private string docImageUrl;
        private string csvImageUrl;
        private string pdfImageUrl;
        private string zipImageUrl;
        private bool excelEnabled = true;
        private bool docEnabled = true;
        private bool csvEnabled = true;
        private bool pdfEnabled = true;
        private bool zipEnabled = true;
        private string reportName;
        private string fileName;
        
        private int imageWidth;
        private int imageHeight;

        private string requestXML;
        private string tableExportXsl;
        private string libraryName;
        private string nameSpace;
        private string methodName;
        private string methodReturnType;
        private bool execMethodWithReflaction;
        private DataTable dataSource;
        #endregion

        #region Properties
        public string RequestXML
        {
            get { return requestXML; }
            set { requestXML = value; }
        }

        public string TableExportXsl
        {
            get { return tableExportXsl; }
            set { tableExportXsl = value; }
        }

        public string ExcelImageUrl
        {
            get { return excelImageUrl; }
            set { excelImageUrl = value; }
        }

        public string ReportName
        {
            get { return reportName; }
            set { reportName = value; }
        }

        public string DocImageUrl
        {
            get { return docImageUrl; }
            set { docImageUrl = value; }
        }

        public string CsvImageUrl
        {
            get { return csvImageUrl; }
            set { csvImageUrl = value; }
        }

        public string PdfImageUrl
        {
            get { return pdfImageUrl; }
            set { pdfImageUrl = value; }
        }

        public string ZipImageUrl
        {
            get { return zipImageUrl; }
            set { zipImageUrl = value; }
        }

        public int ImageWidth
        {
            get { return imageWidth; }
            set { imageWidth = value; }
        }

        public int ImageHeight
        {
            get { return imageHeight; }
            set { imageHeight = value; }
        }

        public string LibraryName
        {
            get { return libraryName; }
            set { libraryName = value; filter.LibraryName = value; }
        }

        public string NameSpace
        {
            get { return nameSpace; }
            set { nameSpace = value; filter.NameSpace = value; }
        }

        public string MethodName
        {
            get { return methodName; }
            set { methodName = value; filter.MethodName = value; }
        }

        public bool ExcelEnabled
        {
            get { return excelEnabled; }
            set { excelEnabled = value; }
        }

        public bool DocEnabled
        {
            get { return docEnabled; }
            set { docEnabled = value; }
        }

        public bool CsvEnabled
        {
            get { return csvEnabled; }
            set { csvEnabled = value; }
        }

        public bool PdfEnabled
        {
            get { return pdfEnabled; }
            set { pdfEnabled = value; }
        }

        public bool ZipEnabled
        {
            get { return zipEnabled; }
            set { zipEnabled = value; }
        }

        public string MethodReturnType
        {
            get { return methodReturnType; }
            set { methodReturnType = value; filter.MethodReturnType = value; }
        }

        public string FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }

        public bool ExecMethodWithReflaction
        {
            get { return execMethodWithReflaction; }
            set { execMethodWithReflaction = value; }
        }

        public DataTable DataSource
        {
            get { return dataSource; }
            set { dataSource = value; }
        }

        [
        Browsable(false),

        DefaultValue(null),
        Description("The content to be shown in each item."),
        PersistenceMode(PersistenceMode.InnerProperty),
        TemplateContainer(typeof(Columns))
        ]
        public virtual Columns Columns
        {
            get { return columns; }
            set { columns = value; }
        }

        public override ControlCollection Controls
        {
            get
            {
                EnsureChildControls();
                return base.Controls;
            }
        }

        #endregion

        #region Methods
        protected override void OnInit(EventArgs e)
        {
            if (imageWidth == 0) imageWidth = 17;
            if (imageHeight == 0) imageHeight = 17;

            base.OnInit(e);
            ExecMethodWithReflaction = true;
            Page.RegisterRequiresControlState(this);
        }

        protected override void CreateChildControls()
        {
            CreateControl();
            ClearChildViewState();
        }

        protected void Add(string name, string url, ExportFileType type)
        {
            TypedImageButton ibtn = new TypedImageButton();
            ibtn.Type = type;
            ibtn.ImageUrl = url;
            ibtn.Width = ImageWidth;
            ibtn.Height = ImageHeight;
            ibtn.AlternateText = ibtn.ToolTip = ibtn.ID = name;
            ibtn.Click += new ImageClickEventHandler(ibtn_Click);

            Controls.Add(ibtn);
        }

        protected virtual void CreateControl()
        {     
            if(excelEnabled) Add("Make excel", excelImageUrl, ExportFileType.xls);
            if(docEnabled) Add("Make doc", docImageUrl, ExportFileType.doc);
            if(csvEnabled) Add("Make csv", csvImageUrl, ExportFileType.csv);
            if(pdfEnabled) Add("Make pdf", pdfImageUrl, ExportFileType.pdf);
            if(zipEnabled) Add("Make zip", zipImageUrl, ExportFileType.zip);
        }

        protected void ibtn_Click(object sender, ImageClickEventArgs e)
        {
            if (Exporting != null)
                Exporting(this, new EventArgs());

            System.Globalization.CultureInfo ci;
            if (Page.Request.UserLanguages != null && Page.Request.UserLanguages.Length != 0)
                ci = new System.Globalization.CultureInfo(Page.Request.UserLanguages[0]);
            else
                ci = new System.Globalization.CultureInfo("en-US");
            string listSeparator = ci.TextInfo.ListSeparator;

            RequestXML = ExportHelper.BuildRequestXML(filter, columns.colFilter);
            try
            {
                ExportHelper.ExportDataToFile(reportName, RequestXML, TableExportXsl, ((TypedImageButton)sender).Type, listSeparator, fileName, DataSource, ExecMethodWithReflaction);
            }
            catch (OutOfMemoryException)
            {
                string s = "";
                if(((TypedImageButton)sender).Type != ExportFileType.csv && ((TypedImageButton)sender).Type != ExportFileType.zip)
                    s = "sau sa exportati folosind csv sau zip";
                Page.ClientScript.RegisterStartupScript(typeof(Page),
                    "OutOfMemoryScript"
                    , "<script language=JavaScript>alert('Dimensiunea fisierului este prea mare! Incercati sa filtrati " + s + ".');</script>");
            }
        }

        public void AddDefinedParameter(object value)
        {
            filter.AddDefinedParameter(ParameterPassingMode.In, Serialize.SerializeObject(value), value.GetType().FullName, value.GetType().Assembly.FullName);
        }

        public void AddDefinedParameter(ParameterPassingMode passingmode, object value)
        {
            filter.AddDefinedParameter(passingmode, Serialize.SerializeObject(value), value.GetType().FullName, value.GetType().Assembly.ToString());
        }

        public void AddSystemParameter(string name, ParameterPassingMode passingmode, string type, string value)
        {
            filter.AddSystemParameter(name, passingmode, type, value);
        }
        #endregion
    }

    public class TypedImageButton : ImageButton
    {
        public ExportFileType Type;
    }

    public class Columns
    {
        #region Member variables

        private Column column = new Column();

        #endregion

        #region Properties

        public ColumnsFilter colFilter = new ColumnsFilter();

        [
        Browsable(false),
        DefaultValue(null),
        Description("The content to be shown in each item."),
        PersistenceMode(PersistenceMode.InnerProperty),
        TemplateContainer(typeof(Column))
        ]
        public virtual Column Column
        {
            get { return column; }
            set { column = value; colFilter.AddColumn(column.DataField, column.DataType, column.HeaderText, column.DataFormatString); }
        }

        public void Add(string columnName)
        {
            colFilter.AddColumn(columnName);
        }

        public void Add(string columnName, string columnHeaderText)
        {
            colFilter.AddColumn(columnName, Column.columnType.Default, columnHeaderText.Replace(" ", "_"), "");
        }

        public void Add(string columnName, string columnHeaderText, Column.columnType columnType)
        {
            colFilter.AddColumn(columnName, columnType, columnHeaderText.Replace(" ", "_"), "");
        }

        public void Add(string columnName, string columnHeaderText, Column.columnType columnType, string dataFormatString)
        {
            colFilter.AddColumn(columnName, columnType, columnHeaderText.Replace(" ", "_"), dataFormatString);
        }

        public void Clear()
        {
            colFilter.Clear();
        }
        #endregion
    }
    
    public class Column
    {
        #region Member variables

        public enum columnType
        {
            Text,
            Number,
            Default 
        }
        private columnType dataType;
        private string dataField;
        private string headerText;
        private string dataFormatString;

        #endregion

        #region Properties
        
        public virtual string DataField
        {
            get { return dataField; }
            set { dataField = value; }
        }

        public virtual columnType DataType
        {
            get { return (columnType)dataType; }
            set { dataType = (columnType)value; }
        }

        public virtual string HeaderText
        {
            get { return headerText; }
            set { headerText = value; }
        }

        public virtual string DataFormatString
        {
            get { return dataFormatString; }
            set { dataFormatString = value; }
        }

        #endregion
    }
}
