﻿using System;
using System.Web.Mvc;
using System.Web;
using System.Text;

namespace Library.Mvc
{

    /// <summary>
    /// LiteGridHelperExtension is used to create a friendly interface for initializing our custom ajax grid
    /// </summary>
    public static class LiteGridHelperExtension
    {
        public static LiteGrid LiteGrid(this HtmlHelper helper, string _id)
        {
            return new LiteGrid(_id);
        }   
    }

    /// <summary>
    /// Build grid on client side
    /// </summary>
    public class LiteGrid
    {
        #region private properties

        private string _id;
        private string _parent;
        private string _exportUrl;
        private string _selectUrl;
        private string _clientFormSelector;
        private string _clientExportSelector;
        private string _clientTemplateSelector;
        private string _defaultSortColumn;
        private string _onSelect;
        private string _onExport;
        private string _onLoad;
        private string _onInitializing;
        private bool _enablePaging;
        private bool _autoLoad;

        #endregion

        #region c-tors

        public LiteGrid(string _id)
        {
            this._id = _id;
            _enablePaging = true;
            _autoLoad = false;
        }

        #endregion

        #region public methods

        public LiteGrid EnablePaging(bool _enablePaging)
        {
            this._enablePaging = _enablePaging;
            return this;
        }

        public LiteGrid AutoLoad(bool _autoLoad)
        {
            this._autoLoad = _autoLoad;
            return this;
        }

        public LiteGrid Export(string exportUrl)
        {
            this._exportUrl = exportUrl;
            return this;
        }

        public LiteGrid Select(string _selectUrl)
        {
            this._selectUrl = _selectUrl;
            return this;
        }

        public LiteGrid Parent(string _parent)
        {
            this._parent = _parent;
            return this;
        }

        public LiteGrid ClientFormSelector(string _clientFormFilter)
        {
            this._clientFormSelector = _clientFormFilter;
            return this;
        }

        public LiteGrid ClientExportSelector(string _clientExportSelector)
        {
            this._clientExportSelector = _clientExportSelector;
            return this;
        }

        public LiteGrid ClientTemplateSelector(string _clientTemplateSelector)
        {
            this._clientTemplateSelector = _clientTemplateSelector;
            return this;
        }

        public LiteGrid DefaultSortColumn(string _defaultSortColumn)
        {
            this._defaultSortColumn = _defaultSortColumn;
            return this;
        }

        public LiteGrid OnSelect(string _onSelect)
        {
            this._onSelect = _onSelect;
            return this;
        }

        public LiteGrid OnExport(string _onExport)
        {
            this._onExport = _onExport;
            return this;
        }

        public LiteGrid OnLoad(string _onLoad)
        {
            this._onLoad = _onLoad;
            return this;
        }

        public LiteGrid OnInitializing(string _onInitializing)
        {
            this._onInitializing = _onInitializing;
            return this;
        }

        public MvcHtmlString Render()
        {
            return MvcHtmlString.Create(ToString());
        }

        /// <summary>
        /// Generates a string script, that will be used to initialize the ajaxgrid plugin
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            // Create javascript
            StringBuilder script = new StringBuilder();

            // Start script
            script.AppendLine("<script type=\"text/javascript\">");

            string onSelect = "";
            if (!string.IsNullOrWhiteSpace(_onSelect))
                onSelect = string.Format(", onSelect : {0} ", _onSelect);

            string onExport = "";
            if (!string.IsNullOrWhiteSpace(_onExport))
                onExport = string.Format(", onExport : {0} ", _onExport);

            string onInitializing = "";
            if (!string.IsNullOrWhiteSpace(_onInitializing))
                onInitializing = string.Format(", onInitializing : {0} ", _onInitializing);

            string onLoad = "";
            if (!string.IsNullOrWhiteSpace(_onLoad))
                onLoad = string.Format(", onLoad : {0} ", _onLoad);

            script.AppendFormat("$('#{0}').ajaxgrid('init', {{ templateSelector: '{1}', defaultSortColumn: '{2}' {3}, selectUrl : '{4}', clientFormSelector : '{5}', clientExportSelector : '{11}', parent : '{12}' {6}, exportUrl : '{7}', enablePaging : {8} {9} {10} }});", _id, _clientTemplateSelector, _defaultSortColumn, onSelect, _selectUrl, _clientFormSelector, onExport, _exportUrl, _enablePaging.ToString().ToLower(), onInitializing, onLoad, _clientExportSelector, _parent).AppendLine();

            if (_autoLoad)
            {
                string bindCall = string.Format("$('#{0}').ajaxgrid('bind');", _id);
                script.Append(bindCall).AppendLine();
            }

            // End script                           
            script.AppendLine("</script>");
            script.AppendLine();

            return script.ToString();
        }

        #endregion
    }
}
