﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.IO.Compression;
using ICSharpCode.SharpZipLib.Zip;
using ICSharpCode.SharpZipLib.Checksums;

namespace Library.Helpers
{
    public class CompressionHelper
    {
        public static byte[] CompressWithZip(string text)
        {
            return CompressWithZip(text, "filekep.txt");
        }

        public static byte[] CompressWithZip(string text, string fileName)
        {
            byte[] data = Encoding.UTF8.GetBytes(text);
            return CompressWithZip(data, fileName);
        }

        /// <param name="levelOfCommpression">Cuprins intre 0 si 9</param>
        public static byte[] CompressWithZip(byte[] data, string fileName)
        {
            byte levelOfCommpression = 9;

            MemoryStream ms = new MemoryStream();
            ZipOutputStream stream = new ZipOutputStream(ms);

            stream.SetLevel(levelOfCommpression);

            ZipEntry entry = new ZipEntry(fileName);
            entry.DateTime = DateTime.Now;
            entry.Size = data.Length;

            Crc32 crc32 = new Crc32();
            crc32.Update(data);
            entry.Crc = crc32.Value;

            stream.PutNextEntry(entry);
            stream.Write(data, 0, data.Length);
            stream.Finish();
            stream.Close();

            return ms.ToArray();
        }

        public static byte[] DecompressWithZip(byte[] compressedData)
        {
            return DecompressWithZip(compressedData, "filekep.txt");
        }

        public static byte[] DecompressWithZip(Stream stream, string fileName)
        {         
            byte[] data = new byte[stream.Length];
            stream.Read(data, 0, data.Length);
            return DecompressWithZip(data, fileName);
        }

        public static byte[] DecompressWithZip(byte[] compressedData, string fileName)
        {
            MemoryStream outputMs = new MemoryStream();
            Ionic.Zip.ZipFile zipFile = Ionic.Zip.ZipFile.Read(compressedData);
            zipFile.Extract(fileName, outputMs);
            return outputMs.ToArray();
        }

        public static byte[] CompressWithGZip(byte[] data)
        {            
            MemoryStream ms = new MemoryStream();
            GZipStream zipStream = new GZipStream(ms, CompressionMode.Compress, true);
            zipStream.Write(data, 0, data.Length);
            zipStream.Close();
                    
            return ms.ToArray();
        }

        public static byte[] CompressWithGZip(string text)
        {
            byte[] data = Encoding.UTF8.GetBytes(text);
            return CompressWithGZip(data);
        }

        public static byte[] DecompressWithGZip(byte[] data)
        {
            MemoryStream input = new MemoryStream();         
            input.Write(data, 0, data.Length);            
            input.Position = 0;            
            GZipStream gzip = new GZipStream(input, CompressionMode.Decompress, true);                        
            MemoryStream output = new MemoryStream();            
            byte[] buff = new byte[64];
            int read = -1;                                    
            read = gzip.Read(buff, 0, buff.Length);            
            while (read > 0)
            {                
                output.Write(buff, 0, read);             
                read = gzip.Read(buff, 0, buff.Length);
            }
            gzip.Close();            
            return output.ToArray();
        }
    }
}
