﻿using System;
using System.Collections.Generic;
using System.Data.Metadata.Edm;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Core.Entities;
using Core.IRepository;
using System.Data.Objects;
using System.Linq.Expressions;
using Core.Queries;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using Core.Messages;
using Library.Extensions;

namespace Repositories
{
    public abstract class BaseRepository<TId, TEntity> : IBaseRepository<TId, TEntity> 
        where TEntity : class
    {
        public DbContext EFContext;

        public Repositories.InsuranceEntities Context
        {
            get { return (Repositories.InsuranceEntities)EFContext; }
        }

        protected abstract DbSet<TEntity> DbSet { get; }        

        public BaseRepository(DbContext EFContext)
        {
            this.EFContext = EFContext;
        }

        public virtual TEntity FindBy(TId id)
        {
            return DbSet.Find(id);              
        }

        public virtual void Add(TEntity entity)
        {
            //DbSet.Attach(entity);
            EFContext.Entry(entity).State = System.Data.EntityState.Added;
            //DbSet.Add(entity);
        }

        public virtual void Edit(TEntity entity)
        {            
            EFContext.Entry(entity).State = System.Data.EntityState.Modified;
        }

        public void Delete(TEntity entity)
        {
            EFContext.Entry(entity).State = System.Data.EntityState.Deleted;
        }

        public void Delete(Expression<Func<TEntity, bool>> predicate)
        {
            var list = DbSet
               .Where(predicate)
               .ToList();

            foreach (TEntity entity in list)
            {                
                EFContext.Entry(entity).State = System.Data.EntityState.Deleted;
            }
        }

        public IList<TEntity> FindBy(Expression<Func<TEntity, bool>> predicate)
        {
            var list = DbSet
                .Where(predicate)
                .ToList();

            return list;
        }

        public virtual ListResponse<TEntity> FindBy(BaseQuery<TEntity> query)
        {            
            ListResponse<TEntity> response = new ListResponse<TEntity>();

            if(query.PagingEnabled && string.IsNullOrEmpty(query.SortColumn))
                throw new Exception("PagingEnabled enabled but SortColumn not defined");

            if (query.PagingEnabled && !string.IsNullOrEmpty(query.SortColumn))
            {
                var q = DbSet.Where(query.ToExpression()).AsQueryable();
                if(query.SortAscending)
                    q = q.OrderBy(query.SortColumn);
                else
                    q = q.OrderByDescending(query.SortColumn);

                response.List =  q.Skip((query.PageNumber - 1) * query.PageSize).Take(query.PageSize)
                                .ToList();

                if (query.ComputePagesCount)
                    response.RowsCount = DbSet.Where(query.ToExpression()).Count();
            }
            else
                response.List = DbSet.Where(query.ToExpression()).ToList();
  
            return response;
        }              
    }

    
}
