﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using Core.IServices;
using Core.Queries;
using Core.Entities;
using Core.IRepository;
using Library.Logging;
using Library.Helpers;
using Core.Messages;

namespace Services
{
    public class ReadBaseService<TId, TEntity, TDto, TQuery> : IReadBaseService<TId, TEntity, TDto, TQuery>
        where TEntity : BaseEntity<TId>
        where TQuery : BaseQuery<TEntity>
    {      
        protected IUnitOfWork UnitOfWork;
        protected IBaseRepository<TId, TEntity> Repository;
        protected ILogger Logger;        
                   
        public ReadBaseService(IUnitOfWork unitOfWork, IBaseRepository<TId, TEntity> Repository, ILogger Logger)
        {
            this.UnitOfWork = unitOfWork;
            this.Repository = Repository;
            this.Logger = Logger;           
        }

        protected virtual TDto MapEntityToDto(TEntity entity)
        {
            return AutoMapper.Mapper.Map<TEntity, TDto>(entity);
        }

        protected virtual void MapForeignRelationsInDto(TEntity entity, TDto dto) { }                

        public virtual BaseResponse<TDto> FindById(string id)
        {
            BaseResponse<TDto> response = new BaseResponse<TDto>();
            try
            {
                TEntity entity = Repository.FindBy(ConvertToIdType(id));
                response.Value = MapEntityToDto(entity);
                MapForeignRelationsInDto(entity, response.Value);
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
                Logger.Log(ex);
            }
            return response;
        }

        public virtual ListResponse<TResult> FindBy<TResult>(TQuery query)
        {
            ListResponse<TResult> response = new ListResponse<TResult>();
            try
            {
                ListResponse<TEntity> dbResponse = Repository.FindBy(query);

                response.List = AutoMapperHelper.MapList<TEntity, TResult>(dbResponse.List).ToList();
                response.RowsCount = dbResponse.RowsCount;
                response.PageNumber = query.PageNumber;
                response.PageSize = query.PageSize;
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
                Logger.Log(ex);
            }
            return response;
        }

        public virtual ListResponse<TDto> FindBy(TQuery query)
        {
            return FindBy<TDto>(query);
        }

        protected TId ConvertToIdType(string id)
        {
            return (TId)Convert.ChangeType(id, typeof(TId));
        }
    }
}
