﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Core.IServices;
using Core.IRepository;
using Core.IGateways;
using Core.Entities;
using Library.StructureMap;
using Library.Helpers;
using Library.Logging;
using System.ServiceModel.Activation;
using Core.Queries;
using Core.Messages;
using Core.Dtos;

namespace Services
{
    [StructureMapBehavior]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class PolicyService : CrudBaseService<int, Policy, PolicyDto, PolicyQuery>, IPolicyService
    {
        IPolicyRepository PolicyRepository;
        IPersonRepository PersonRepository;
        IPolicyReport PolicyReport;
        
        public PolicyService(IUnitOfWork UnitOfWork,ILogger Logger, IPolicyRepository PolicyRepository, IPersonRepository PersonRepository, IPolicyReport PolicyReport) :base(UnitOfWork, PolicyRepository, Logger) 
        {
            this.PolicyRepository = PolicyRepository;
            this.PersonRepository = PersonRepository;
            this.PolicyReport = PolicyReport;
        }

        protected override PolicyDto MapEntityToDto(Policy entity)
        {
            return entity.ToDtoInheritance();
        }

        protected override Policy MapDtoToEntity(PolicyDto dto)
        {
            return dto.ToEntityInheritance();
        }

        protected override Policy MapDtoToEntity(PolicyDto dto, Policy entity)
        {
            return dto.ToEntityInheritance(entity);
        }

        protected override void EditForeignRelations(Policy entity, PolicyDto dto)
        {
            string personName = dto.Person.Name;
            Person person = null;

            person = PersonRepository.FindBy(x => x.Name.Equals(personName)).FirstOrDefault();
            if (person != null)
            {
                entity.Person = person;
                entity.PersonID = person.Id;
            }
            else
            {
                person = dto.Person.ToEntity();
                PersonRepository.Add(person);
                entity.Person = person;
                entity.PersonID = person.Id;
            }
        }

        protected override void MapForeignRelationsInDto(Policy entity, PolicyDto dto)
        {            
            dto.Person = entity.Person.ToDto();
        }

        public System.IO.Stream GenerateDocument(string id)
        {
            Policy policy = PolicyRepository.FindBy(Convert.ToInt32(id));
            PolicyReport.Parameters["policy"] = policy;
            byte[] content = PolicyReport.GetPdf();
            Stream fileStream = StreamHelper.CreateStreamFromByteArray(content);
            return fileStream;
        }
    }
}