﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.IRepository;
using System.Data.Objects;
using Lib.Query;
using Lib.Query.EntityFramework;

namespace DataAccess.EntityFramework
{
    public abstract class BaseRepository<TId, TEntity> : IBaseRepository<TId, TEntity> where TEntity : class
    {
        protected abstract ObjectSet<TEntity> ObjectSet { get; }

        public InsuranceSampleEntities Context { get { return (InsuranceSampleEntities)EFContext; } }
        public ObjectContext EFContext;

        public BaseRepository(ObjectContext EFContext)
        {
            this.EFContext = EFContext;
            EFContext.ContextOptions.ProxyCreationEnabled = false; //for WCF         
        }
        
        protected string EntitySetName
        {
            get
            {
                return ObjectSet.EntitySet.Name;
            }
        }

        public virtual TEntity FindBy(TId id)
        {
            var keyPropertyName = ObjectSet.EntitySet.ElementType.KeyMembers[0].ToString();
            TEntity entity = ObjectSet.Where("it." + keyPropertyName + "=" + id).First();
            return entity;
        }
      
        public virtual IList<TEntity> FindBy(IReadLiteQuery query)
        {
            return query.Translate(ObjectSet).ToList();
        }

        public virtual TId Add(TEntity entity)
        {
            EFContext.AddObject(EntitySetName, entity);
            EFContext.SaveChanges();
            return (TId)EFContext.CreateEntityKey(EntitySetName, entity).EntityKeyValues[0].Value;            
        }

        public virtual void Edit(TEntity entity)
        {
            EFContext.SaveChanges();
        }

        public void Delete(TEntity entity)
        {
            EFContext.DeleteObject(entity);
            EFContext.SaveChanges();
        }
    } 
}
