﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Library.UnitOfWork;
using NHibernate;
using Lib.Query;
using Lib.Query.NHibernate;

namespace DataAccess.NHibernate
{
    public abstract class BaseRepository<TId, TEntity> where TEntity:class
    {
        protected IUnitOfWork _uow;
        protected ISession NHContext;

        public BaseRepository(IUnitOfWork uow, ISession nHContext)           
        {
            this._uow = uow;
            this.NHContext = nHContext;
        }

        public TId Add(TEntity entity)
        {
            object id = NHContext.Save(entity);
            NHContext.Flush();
            return (TId)id;
        }

        public void Delete(TEntity entity)
        {
            NHContext.Delete(entity);
            NHContext.Flush();
        }

        public void Edit(TEntity entity)
        {                    
            //NHContext.Merge(entity);        
            NHContext.SaveOrUpdate(entity);
            NHContext.Flush();
        }

        public TEntity FindBy(TId id)
        {
            return NHContext.Get<TEntity>(id);
        }

        public IList<TEntity> FindBy(IReadLiteQuery query)
        {
            ICriteria criteriaQuery = NHContext.CreateCriteria(typeof(TEntity));
            query.Translate<TEntity>(criteriaQuery);
            return criteriaQuery.List<TEntity>();
        }
    }
}
