﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Library.Messages;
using Core.IServices;
using Core.Queries;
using Core;
using Core.IGateways;
using System.IO;
using Library.Helpers;

namespace Policies.App2.Controllers
{
    [Authorize(Roles = "Agent")]
    public class PolicyController : Controller
    {
        IPolicyService PolicyService;              

        public PolicyController(IPolicyService PolicyService)
        {
            this.PolicyService = PolicyService;        
        }

        public ActionResult Find()
        {
            PolicyQuery query = new PolicyQuery();
            ListResponse<Policy> response = PolicyService.FindBy(query);
            return View("Find", response.List);
        }

        [HttpPost]
        public ActionResult Find(PolicyQuery query)
        {
            ListResponse<Policy> response = PolicyService.FindBy(query);
            return View("Find", response.List);
        }

        public ActionResult Details(string id)
        {
            BaseResponse<Policy> response = PolicyService.FindById(id);
            return View("Details", response.Value);
        }

        public ActionResult Create()
        {
            return View("Create");
        }

        [HttpPost]
        public ActionResult Create(Core.MotorPolicy policy, Core.Person person)
        {            
            BaseResponse<int> response = null;
            if (ModelState.IsValid)
            {
                policy.Person = person;
                response = PolicyService.Add(policy);
                ViewBag.Response = response;

                if (response.Success)
                {
                    BaseResponse<Policy> detailsResponse = PolicyService.FindById(response.Value.ToString());
                    return View("Details", detailsResponse.Value);
                }
                else
                    return View("Create", policy);
            }
            else
                return View("Create", policy);
        }

        public ActionResult Edit(string id)
        {
            BaseResponse<Policy> response = PolicyService.FindById(id);
            return View("Edit", response.Value);
        }

        public ActionResult ConfirmDelete(string id)
        {
            BaseResponse<Policy> response = PolicyService.FindById(id);
            return View("Delete", response.Value);
        }

        [HttpPost]
        public ActionResult Delete(string id)
        {
            BaseResponse response = PolicyService.Delete(id);
            ViewBag.Response = response;
            return View("DeleteStatus", (object)id);
        }  
        
        [HttpPost]
        public ActionResult Edit(Core.MotorPolicy policy, Core.Person person)
        {
            BaseResponse response = null;
            if (ModelState.IsValid)
            {
                //build policy from request form
                policy.Person = person;
                policy.Person.Id = policy.PersonID;

                response = PolicyService.Edit(policy);
                ViewBag.Response = response;

                if (response.Success)
                {
                    BaseResponse<Policy> detailsResponse = PolicyService.FindById(policy.Id.ToString());
                    return View("Details", detailsResponse.Value);
                }
                else
                    return View("Edit", policy);
            }
            else
                return View("Edit", policy);
        }        

        public FileStreamResult Document(string id)
        {
            Stream fileStream = PolicyService.GenerateDocument(id);
            HttpContext.Response.AddHeader("content-disposition","attachment; filename=policy.pdf");
            return new FileStreamResult(fileStream, "application/pdf");
        }
    }
}
