﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using StructureMap;
using Library.Logging;
using Services;
using Core;
using StructureMap.Pipeline;

namespace Factory
{ 
    public static class Bootstrapper
    {
        public static void Init(bool useProxyServices, OrmType ormType, InstanceScope ormContextLifecycle)
        {
            ObjectFactory.Initialize(x =>
            {                                                                                        
                switch (ormType)
                {
                    case OrmType.EntityFramework:
                        x.AddRegistry(new EntityFrameworkRegistry(ormContextLifecycle));  
                        break;
                    case OrmType.NHibernate:
                        x.AddRegistry(new NHibernateRegistry(ormContextLifecycle));
                        break;
                    case OrmType.None:
                        break;
                }
                if(useProxyServices)                     
                    x.AddRegistry<ProxyServiceRegistry>();
                else
                    x.AddRegistry<ServiceRegistry>();

                x.For<ILogger>().Singleton().Use<Log4NetAdapter>();
            });
            
            AutoMapper.Mapper.CreateMap<MotorPolicy, MotorPolicy>().ForMember(x => x.Person, opt => opt.Ignore());
        }

        public enum OrmType
        {
            None,
            EntityFramework,
            NHibernate
        }
    }
}
