﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Core.IServices;
using Core.IRepository;
using Library.Messages;
using Lib.Query;
using Core;
using Library.StructureMap;
using Core.IGateways;
using System.IO;
using Library.Helpers;
using System.ServiceModel.Activation;

namespace Services
{
    [StructureMapBehavior]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class PolicyService : IPolicyService
    {
        IPolicyRepository PolicyRepository;
        IPersonRepository PersonRepository;
        IPolicyReport PolicyReport;

        public PolicyService(IPolicyRepository PolicyRepository, IPersonRepository PersonRepository, IPolicyReport PolicyReport)
        {
            this.PolicyRepository = PolicyRepository;
            this.PersonRepository = PersonRepository;
            this.PolicyReport = PolicyReport;
        }

        public BaseResponse<Policy> FindById(string id)
        {
            BaseResponse<Policy> response = new BaseResponse<Policy>();
            try
            {
                response.Value = PolicyRepository.FindBy(int.Parse(id));
                if (response.Value.Person == null)
                    response.Value.Person = PersonRepository.FindBy(response.Value.PersonID); //WCF ProxyCreationEnabled
                
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
            }
            return response;            
        }

        public ListResponse<Policy> FindBy(Core.Queries.PolicyQuery query)
        {
            ListResponse<Policy> response = new ListResponse<Policy>();
            try
            {
                response.List = PolicyRepository.FindBy(query.ToLiteQuery());                
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
            }
            return response;            
        }

        public BaseResponse<int> Add(Core.Policy request)
        {
            BaseResponse<int> response = new BaseResponse<int>();
            try
            {
                LiteQuery<Person> q = new LiteQuery<Person>();
                q.Add(x => x.Name, CriteriaOperator.Equal, request.Person.Name);
                IList<Person> personList = PersonRepository.FindBy(q);
                if (personList.Count > 0)
                {
                    request.Person = personList.FirstOrDefault();
                    request.PersonID = request.Person.Id;
                }

                response.Value = PolicyRepository.Add(request);
                response.Success = true;                
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
            }
            
            return response;
        }

        public Library.Messages.BaseResponse Edit(Core.Policy request)
        {
            BaseResponse response = new BaseResponse();
            try
            {
                Policy policy = PolicyRepository.FindBy(request.Id);
                if(policy.Person == null)
                    policy.Person = PersonRepository.FindBy(policy.PersonID); //WCF ProxyCreationEnabled

                AutoMapper.Mapper.Map(request, policy, request.GetType(), policy.GetType());
                policy.Person.Name = request.Person.Name;

                PolicyRepository.Edit(policy);
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
            }

            return response;
        }

        public Library.Messages.BaseResponse Delete(string id)
        {
            BaseResponse response = new BaseResponse();
            try
            {
                Policy policy = PolicyRepository.FindBy(int.Parse(id));
                PolicyRepository.Delete(policy);
                response.Success = true;
            }
            catch (Exception ex)
            {
                response.Success = false;
                response.ErrorMessage = ex.ToString();
            }

            return response;
        }

        public System.IO.Stream GenerateDocument(string id)
        {
            BaseResponse<Policy> response = FindById(id);            
            PolicyReport.Parameters["policy"] = response.Value;
            byte[] content = PolicyReport.GetPdf();
            Stream fileStream = StreamHelper.CreateStreamFromByteArray(content);
            return fileStream;
        }
    }
}
