﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Policies.App2.Controllers;
using System.Web.Mvc;
using Core;
using StructureMap;
using Factory;
using Library.Messages;
using Core.Queries;

namespace Tests.Controllers
{
    [TestClass]
    public class PolicyControllerTest
    {
        [TestInitialize()]
        public void Init()
        {            
        }

        [TestMethod]
        public void CRUDProxy()
        {
            Bootstrapper.Init(true, Bootstrapper.OrmType.None, InstanceScope.Singleton);
            CRUD();
        }

        [TestMethod]
        public void CRUDEntityFramework()
        {
            Bootstrapper.Init(false, Bootstrapper.OrmType.EntityFramework, InstanceScope.Singleton);
            CRUD();
        }

        [TestMethod]
        public void CRUDNHibernate()
        {
            Bootstrapper.Init(false, Bootstrapper.OrmType.NHibernate, InstanceScope.Singleton);
            CRUD();
        }
        
        public void CRUD()
        {
            int count;                        

            string id = Create();
            Read(id);
            Policy policy = Edit(id);

            PolicyQuery query = new PolicyQuery();
            query.Number = policy.Number.ToString();
            count = Find(query);
            Assert.AreEqual(true, (count > 0));


            int countBefore = Find(query);
            Delete(id);

            count = Find(query);
            Assert.AreEqual(true, (countBefore > count));
        }

        public string Create()
        {
            // Arrange            
            PolicyController controller = ObjectFactory.GetInstance<PolicyController>();
            MotorPolicy policy = GetTestPolicy(null);

            // Act            
            ViewResult result = controller.Create(policy, policy.Person) as ViewResult;

            // Assert
            Assert.IsNotNull(result);
            Assert.AreEqual(true, ((BaseResponse)result.ViewBag.Response).Success);

            Assert.AreEqual("Details", result.ViewName);
            Assert.AreEqual(true, result.ViewData.Model is MotorPolicy);
            Assert.AreEqual(policy.Number, ((MotorPolicy)result.ViewData.Model).Number);
            Assert.AreEqual(policy.PlateNo, ((MotorPolicy)result.ViewData.Model).PlateNo);
            Assert.AreEqual(policy.Person.Name, ((MotorPolicy)result.ViewData.Model).Person.Name);

            return ((MotorPolicy)result.ViewData.Model).Id.ToString();
        }

        public void Read(string id)
        {
            // Arrange            
            PolicyController controller = ObjectFactory.GetInstance<PolicyController>();
            MotorPolicy policy = GetTestPolicy(null);

            // Act            
            ViewResult result = controller.Details(id) as ViewResult;

            // Assert
            Assert.IsNotNull(result);            
            Assert.AreEqual("Details", result.ViewName);
            Assert.AreEqual(true, result.ViewData.Model is MotorPolicy);
            Assert.AreEqual(policy.Number, ((MotorPolicy)result.ViewData.Model).Number);
            Assert.AreEqual(policy.PlateNo, ((MotorPolicy)result.ViewData.Model).PlateNo);
            Assert.AreEqual(policy.Person.Name, ((MotorPolicy)result.ViewData.Model).Person.Name);
        }

        public MotorPolicy Edit(string id)
        {
            // Arrange            
            PolicyController controller = ObjectFactory.GetInstance<PolicyController>();
                                             
            // Act            
            ViewResult result;
            result = controller.Edit(id) as ViewResult;
            MotorPolicy policy = (MotorPolicy)result.ViewData.Model;
            policy.PlateNo = "C54321";
            policy.Person.Name = "54321P";
            result = controller.Edit(policy, policy.Person) as ViewResult;

            // Assert
            Assert.IsNotNull(result);
            Assert.AreEqual(true, ((BaseResponse)result.ViewBag.Response).Success);
            Assert.AreEqual("Details", result.ViewName);
            Assert.AreEqual(true, result.ViewData.Model is MotorPolicy);
            Assert.AreEqual(policy.Number, ((MotorPolicy)result.ViewData.Model).Number);
            Assert.AreEqual(policy.PlateNo, ((MotorPolicy)result.ViewData.Model).PlateNo);
            Assert.AreEqual(policy.Person.Name, ((MotorPolicy)result.ViewData.Model).Person.Name);

            return (MotorPolicy)result.ViewData.Model;
        }

        public void Delete(string id)
        {
            // Arrange            
            PolicyController controller = ObjectFactory.GetInstance<PolicyController>();            

            // Act            
            ViewResult result = controller.Delete(id) as ViewResult;

            // Assert
            Assert.IsNotNull(result);
            Assert.AreEqual(true, ((BaseResponse)result.ViewBag.Response).Success);
        }

        public int Find(PolicyQuery query)
        {
            // Arrange            
            PolicyController controller = ObjectFactory.GetInstance<PolicyController>();         

            // Act            
            ViewResult result = controller.Find(query) as ViewResult;

            // Assert
            Assert.IsNotNull(result);
            Assert.AreEqual("Find", result.ViewName);            

            return ((IEnumerable<Policy>)result.ViewData.Model).Count<Policy>();
        }

        public MotorPolicy GetTestPolicy(string id)
        {
            MotorPolicy policy = new MotorPolicy();
            if(id != null)
                policy.Id = int.Parse(id);

            policy.Number = 1234566;
            policy.PlateNo = "C1234566";

            Person person = new Person();
            person.Name = "C1234566";

            policy.Person = person;
            return policy;
        }
    }
}
